// Learn cc.Class:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/class.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/class.html
// Learn Attribute:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/life-cycle-callbacks.html


const GameCamera = require('GameCamera');
cc.director.getPhysicsManager().debugDrawFlags = 0;

cc.Class({
   extends: cc.Component,

   properties: {
	   camera: GameCamera,
	   jumpHeight: 0,
	   jumpDuration: 0,
	   jumpSpeed: 200,
	   jumps: 1,
	   minMoveSpeed: 0,
	   maxMoveSpeed: 0,
	   accel: 0,
   },
   
   onLoad: function () {
        // initialize jump action
		 // switch of acceleration direction
        this.accLeft = false;
        this.accRight = false;
		this.oneJump = false;
		
		this.body = this.getComponent(cc.RigidBody);

        // current horizontal speed of main character
        this.xSpeed = 0;
		this.node.zIndex = 2;
		
		this.setInputControl();
		
    },
   
    setJumpAction: function () {
        // jump up
        var jumpUp = cc.moveBy(this.jumpDuration, cc.v2(0, this.jumpHeight)).easing(cc.easeCubicActionOut());
        // jump down
        var jumpDown = cc.moveBy(this.jumpDuration, cc.v2(0, -this.jumpHeight)).easing(cc.easeCubicActionIn());
        // repeat
        return cc.repeat(cc.sequence(jumpUp, jumpDown), 1);
    },
	
		
	 setInputControl: function () {
        var self = this;
        // add keyboard event listener
        // When there is a key being pressed down, judge if it's the designated directional button and set up acceleration in the corresponding direction
        cc.systemEvent.on(cc.SystemEvent.EventType.KEY_DOWN, function (event){
            switch(event.keyCode) {
                case cc.KEY.a:
				case cc.KEY.left:
                    self.accLeft = true;
                    break;
                case cc.KEY.d:
				case cc.KEY.right:
                    self.accRight = true;				
                    break;
				case cc.KEY.w:
				case cc.KEY.up:
                    self.oneJump = true;
					this.jumps--;
				    break;
            }
        });

        // when releasing the button, stop acceleration in this direction
        cc.systemEvent.on(cc.SystemEvent.EventType.KEY_UP, function (event){
            switch(event.keyCode) {
                case cc.KEY.a:
				case cc.KEY.left:
                    self.accLeft = false;
                    break;
                case cc.KEY.d:
				case cc.KEY.right:
                    self.accRight = false;
                    break;
				case cc.KEY.w:
				case cc.KEY.up:
				    self.oneJump = false;
				    break;
            }
		});
    },
	
	
	onBeginContact: function (contact, selfCollider, otherCollider) {
		//check if player is below or on course with obstacles
        if (otherCollider.node.y >= selfCollider.node.y) {	
		   switch(otherCollider.node.name) {
			  case "Block":
		        this.maxMoveSpeed = 0;
				break;
			  case "Ground":
				this.jumps = 1;
				break;
			  }
			  
		} else if (!(otherCollider.node.y >= selfCollider.node.y)) {
			switch(otherCollider.node.name) {
              case "Ground":
			  case "Block":
				this.jumps = 1;
				this.maxMoveSpeed = 100;
				break;
			}
		}
		},
		
	update: function (dt) {
		this.body = this.getComponent(cc.RigidBody);
        // update speed of each frame according to the current acceleration direction
        if (this.accLeft) {
            this.xSpeed -= this.accel * dt;
        } else if (this.accRight) {
            this.xSpeed += this.accel * dt;
        } else if (this.oneJump) {
		    this.jumpAction = this.setJumpAction();
            this.node.runAction(this.jumpAction);
			cc.warn(this.jumps);
		}

		this.body.linearVelocity = cc.v2(this.xSpeed);
		//this.maxMoveSpeed = this.body.linearVelocity;
		
        // restrict the movement speed of the main character to the maximum movement speed
        if ( Math.abs(this.xSpeed) > this.maxMoveSpeed ) {
            // if speed reaches its limit, use the max speed with current direction
            this.xSpeed = this.maxMoveSpeed * this.xSpeed / Math.abs(this.xSpeed);
			//this.body.linearDamping = 100;
        } 
		// restrict speed of main character to minimum mvmt speed
		if ( Math.abs(this.body.linearVelocity) < this.minMoveSpeed ) {
			this.xSpeed = this.minMoveSpeed / this.body.linearVelocity * Math.abs(this.body.linearVelocity);
			//this.body.linearDamping = 0;
		}
		
		// restrict the number of jumps of the character 
		if (this.node.y >= this.jumpHeight) {
			this.oneJump = false;
            //this.xSpeed = this.jumpSpeed;
		}
		
		if (this.jumps <= 0) {
				this.oneJump = false;
			}
		// restrict the jump height of the character 

        // update the position of the main character according to the current speed
        this.node.x += this.xSpeed * dt;
		//this.node.y += this.oneJump * dt;
		//this.body.linearVelocity = this.speed;
    }
});